/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#include <glib-object.h>

#include "../otak-util/otstring.h"
#include "otdbconnection.h"


static gpointer parent_class = NULL;



static void
ot_db_connection_dispose (GObject *gobject)
{
    OtDbConnection *connection = OT_DB_CONNECTION (gobject);

    if (connection->url != NULL)
    {
        g_free (connection->username);
        connection->username = NULL;
    }

    if (connection->username != NULL)
    {
        g_free (connection->username);
        connection->username = NULL;
    }

    if (connection->password != NULL)
    {
        g_free (connection->password);
        connection->password = NULL;
    }

    if (connection->encoding != NULL)
    {
        g_free (connection->encoding);
        connection->encoding = NULL;
    }

    G_OBJECT_CLASS (parent_class)->dispose (gobject);
}

static void
ot_db_connection_base_class_init (OtDbConnectionClass *class)
{
}

static void
ot_db_connection_base_class_finalize (OtDbConnectionClass *class)
{
}

static void
ot_db_connection_class_init (OtDbConnectionClass *class)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (class);

    parent_class = g_type_class_ref (G_TYPE_OBJECT);

    gobject_class->dispose = ot_db_connection_dispose;

    class->new_clone                = NULL;
    class->connect                  = NULL;
    class->disconnect               = NULL;
    class->commit                   = NULL;
    class->rollback                 = NULL;
    class->is_connected             = NULL;
    class->test_connection          = NULL;
    class->get_conn_handle          = NULL;
    class->is_error                 = NULL;
    class->init_error               = NULL;
    class->dup_current_err_msg      = NULL;
    class->dup_table_name_list      = NULL;
    class->dup_index_name_list      = NULL;
    class->dup_procedure_name_list  = NULL;
    class->dup_column_list          = NULL;
    class->dup_index_key_list       = NULL;
    class->dup_simple_sql_select    = NULL;
    class->dup_simple_sql_count     = NULL;
    class->dup_simple_sql_delete    = NULL;
    class->new_dataset              = NULL;
    class->new_column               = NULL;
}

static void
ot_db_connection_init (OtDbConnection       *connection,
                       OtDbConnectionClass  *class)
{
    connection->url         = NULL;
    connection->username    = NULL;
    connection->password    = NULL;
    connection->encoding    = g_strdup (OT_DB_DEFAULT_ENCODING);
}

GType
ot_db_connection_get_type (void)
{
    static GType object_type = 0;

    if (!object_type)
    {
        const GTypeInfo object_info =
        {
            sizeof (OtDbConnectionClass),
            (GBaseInitFunc) ot_db_connection_base_class_init,
            (GBaseFinalizeFunc) ot_db_connection_base_class_finalize,
            (GClassInitFunc) ot_db_connection_class_init,
            NULL,       /* class_finalize */
            NULL,       /* class_data */
            sizeof (OtDbConnection),
            16,     /* n_preallocs */
            (GInstanceInitFunc) ot_db_connection_init,
            NULL,       /* value_table */
        };

        object_type = g_type_register_static (G_TYPE_INITIALLY_UNOWNED, "OtDbConnection", 
                &object_info, G_TYPE_FLAG_ABSTRACT);
    }

    return object_type;
}



/*
 * public
 */

void
ot_db_connection_set_url (OtDbConnection    *connection,
                          const gchar       *url)
{
    if (connection->url != NULL)
    {
        g_free (connection->url);
    }
    
    connection->url = g_strdup (url);
}

void
ot_db_connection_set_username (OtDbConnection   *connection,
                               const gchar      *username)
{
    if (connection->username != NULL)
    {
        g_free (connection->username);
    }
    
    connection->username = g_strdup (username);
}

void
ot_db_connection_set_password (OtDbConnection   *connection,
                               const gchar      *password)
{
    if (connection->password != NULL)
    {
        g_free (connection->password);
    }
    
    connection->password = g_strdup (password);
}

void
ot_db_connection_set_encoding (OtDbConnection   *connection,
                               const gchar      *encoding)
{
    gchar *encoding_tmp = ot_string_dup_trim (encoding);

    if (!ot_string_is_empty (encoding_tmp))
    {
        if (connection->encoding != NULL)
        {
            g_free (connection->encoding);
        }
    
        connection->encoding = g_strdup (encoding_tmp);
    }

    g_free (encoding_tmp);
    encoding_tmp = NULL;
}

OtDbConnection*
ot_db_connection_new_clone (OtDbConnection *connection)
{
    OtDbConnectionClass *class = OT_DB_CONNECTION_GET_CLASS (connection);

    return class->new_clone (connection);
}

gboolean
ot_db_connection_connect (OtDbConnection *connection)
{
    OtDbConnectionClass *class = OT_DB_CONNECTION_GET_CLASS (connection);

    return class->connect (connection);
}

gboolean
ot_db_connection_disconnect (OtDbConnection *connection)
{
    OtDbConnectionClass *class = OT_DB_CONNECTION_GET_CLASS (connection);

    return class->disconnect (connection);
}

gboolean
ot_db_connection_commit (OtDbConnection *connection)
{
    OtDbConnectionClass *class = OT_DB_CONNECTION_GET_CLASS (connection);

    return class->commit (connection);
}

gboolean
ot_db_connection_rollback (OtDbConnection *connection)
{
    OtDbConnectionClass *class = OT_DB_CONNECTION_GET_CLASS (connection);

    return class->rollback (connection);
}

gboolean
ot_db_connection_is_connected (OtDbConnection *connection)
{
    OtDbConnectionClass *class = OT_DB_CONNECTION_GET_CLASS (connection);

    return class->is_connected (connection);
}

gboolean
ot_db_connection_test_connection (OtDbConnection *connection)
{
    OtDbConnectionClass *class = OT_DB_CONNECTION_GET_CLASS (connection);

    return class->test_connection (connection);
}

gpointer
ot_db_connection_get_conn_handle (OtDbConnection *connection)
{
    OtDbConnectionClass *class = OT_DB_CONNECTION_GET_CLASS (connection);

    return class->get_conn_handle (connection);
}

gboolean
ot_db_connection_is_error (OtDbConnection *connection)
{
    OtDbConnectionClass *class = OT_DB_CONNECTION_GET_CLASS (connection);

    return class->is_error (connection);
}

void
ot_db_connection_init_error (OtDbConnection *connection)
{
    OtDbConnectionClass *class = OT_DB_CONNECTION_GET_CLASS (connection);

    return class->init_error (connection);
}

gchar*
ot_db_connection_dup_current_err_msg (OtDbConnection *connection)
{
    OtDbConnectionClass *class = OT_DB_CONNECTION_GET_CLASS (connection);

    return class->dup_current_err_msg (connection);
}

GPtrArray*
ot_db_connection_dup_table_name_list (OtDbConnection    *connection,
                                      OtDbTableType      table_type)
{
    OtDbConnectionClass *class = OT_DB_CONNECTION_GET_CLASS (connection);

    return class->dup_table_name_list (connection, table_type);
}

GPtrArray*
ot_db_connection_dup_index_name_list (OtDbConnection *connection)
{
    OtDbConnectionClass *class = OT_DB_CONNECTION_GET_CLASS (connection);

    return class->dup_index_name_list (connection);
}

GPtrArray*
ot_db_connection_dup_procedure_name_list (OtDbConnection *connection)
{
    OtDbConnectionClass *class = OT_DB_CONNECTION_GET_CLASS (connection);

    return class->dup_procedure_name_list (connection);
}

GPtrArray*
ot_db_connection_dup_column_list (OtDbConnection     *connection,
                                  const gchar        *table_name)
{
    OtDbConnectionClass *class = OT_DB_CONNECTION_GET_CLASS (connection);

    return class->dup_column_list (connection, table_name);
}

GPtrArray*
ot_db_connection_dup_index_key_list (OtDbConnection *connection,
                                     const gchar    *table_name)
{
    OtDbConnectionClass *class = OT_DB_CONNECTION_GET_CLASS (connection);

    return class->dup_index_key_list (connection, table_name);
}

gchar*
ot_db_connection_dup_simple_sql_select (OtDbConnection  *connection,
                                        const gchar     *table_name,
                                        GPtrArray       *fields,
                                        GPtrArray       *orders)
{
    OtDbConnectionClass *class = OT_DB_CONNECTION_GET_CLASS (connection);

    return class->dup_simple_sql_select (connection, table_name, fields, orders);
}

gchar*
ot_db_connection_dup_simple_sql_count (OtDbConnection   *connection,
                                       const gchar      *table_name,
                                       GPtrArray        *where_fields)
{
    OtDbConnectionClass *class = OT_DB_CONNECTION_GET_CLASS (connection);

    return class->dup_simple_sql_count (connection, table_name, where_fields);
}

gchar*
ot_db_connection_dup_simple_sql_delete (OtDbConnection  *connection,
                                        const gchar     *table_name,
                                        GPtrArray       *where_fields)
{
    OtDbConnectionClass *class = OT_DB_CONNECTION_GET_CLASS (connection);

    return class->dup_simple_sql_delete (connection, table_name, where_fields);
}

gpointer
ot_db_connection_new_dataset (OtDbConnection *connection)
{
    OtDbConnectionClass *class = OT_DB_CONNECTION_GET_CLASS (connection);

    return class->new_dataset (connection);
}

OtDbColumn*
ot_db_connection_new_column (OtDbConnection *connection)
{
    OtDbConnectionClass *class = OT_DB_CONNECTION_GET_CLASS (connection);

    return class->new_column (connection);
}

gchar*
ot_db_connection_dup_str_for_view (OtDbConnection   *connection,
                                   const gchar      *str)
{
    return g_convert (str, -1, "UTF-8", connection->encoding, NULL, NULL, NULL);
}

gchar*
ot_db_connection_dup_str_for_dbms (OtDbConnection   *connection,
                                   const gchar      *str)
{
    return g_convert (str, -1, connection->encoding, "UTF-8", NULL, NULL, NULL);
}
