/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#include <glib-object.h>

#include "../otak-util/otstring.h"
#include "otdbdataset.h"


static gpointer parent_class = NULL;



static void
ot_db_dataset_dispose (GObject *gobject)
{
    OtDbDataset *dataset = OT_DB_DATASET (gobject);

    ot_db_dataset_close (dataset);

    G_OBJECT_CLASS (parent_class)->dispose (gobject);
}

static void
ot_db_dataset_base_class_init (OtDbDatasetClass *class)
{
}

static void
ot_db_dataset_base_class_finalize (OtDbDatasetClass *class)
{
}

static void
ot_db_dataset_class_init (OtDbDatasetClass *class)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (class);

    parent_class = g_type_class_ref (G_TYPE_OBJECT);

    gobject_class->dispose = ot_db_dataset_dispose;

    class->prepare              = NULL;
    class->set_param            = NULL;
    class->execute              = NULL;
    class->close                = NULL;
    class->result_next          = NULL;
    class->dup_result_data      = NULL;
    class->update_result_datas  = NULL;
    class->delete_result_datas  = NULL;
    class->get_result_dbtype    = NULL;
}

static void
ot_db_dataset_init (OtDbDataset         *dataset,
                    OtDbDatasetClass    *class)
{
    dataset->connection         = NULL;
    dataset->fetch_size         = 0;
    dataset->req_columns        = NULL;
    dataset->req_stmt_type      = OT_DB_STMT_TYPE_NONE;
    dataset->req_modify_count   = 0;
    dataset->req_cursor_index   = -1;
}

GType
ot_db_dataset_get_type (void)
{
    static GType object_type = 0;

    if (!object_type)
    {
        const GTypeInfo object_info =
        {
            sizeof (OtDbDatasetClass),
            (GBaseInitFunc) ot_db_dataset_base_class_init,
            (GBaseFinalizeFunc) ot_db_dataset_base_class_finalize,
            (GClassInitFunc) ot_db_dataset_class_init,
            NULL,       /* class_finalize */
            NULL,       /* class_data */
            sizeof (OtDbDataset),
            16,     /* n_preallocs */
            (GInstanceInitFunc) ot_db_dataset_init,
            NULL,       /* value_table */
        };

        object_type = g_type_register_static (G_TYPE_INITIALLY_UNOWNED, "OtDbDataset", 
                &object_info, G_TYPE_FLAG_ABSTRACT);
    }

    return object_type;
}



/*
 * public
 */

/*
 * public
 */

void
ot_db_dataset_set_connection (OtDbDataset       *dataset,
                              OtDbConnection    *connection)
{
    dataset->connection = connection;
}

void
ot_db_dataset_set_fetch_size (OtDbDataset   *dataset,
                              gint           fetch_size)
{
    dataset->fetch_size = fetch_size;
}

gboolean
ot_db_dataset_prepare (OtDbDataset  *dataset,
                       const gchar  *query)
{
    OtDbDatasetClass *class = OT_DB_DATASET_GET_CLASS (dataset);

    return class->prepare (dataset, query);
}

/*
 * ot_db_dataset_set_param:
 * @dataset: dataset
 * @index: 바인딩할 파라미터의 0부터 시작하는 index
 * @ctype: C에서 사용한 타입 (Otak Database Lib에서 별도로 재정의함)
 * @vtype: DBMS에서의 타입 (일반적인 타입이며 프로바이더별 구현)
 * @value: 값의 포인터. 값 복사를 하지않고 포인터만 저장하므로 값이 다른 파라미터는 각각의 메모리 공간을 사용해야한다.
 * @length: 버퍼등의 길이를 지정한다. 필수는 아니며 프로바이더가 필요한경우 옵션으로 사용
 *
 * prepare한 쿼리의 파라미터를 바인딩한다. 값 복사를 하지 않고 포인터만 저장하므로 execute전까지는 해당 값들의
 * 메모리를 보존해야한다. execute전에 메모리를 제거할경우 프로바이더의 특성에 따라 잘못된 동작을 할수 있다.
 * 프로바이더마다 조금씩 차이는 있을수 있으나 포인터만의 전달, 그리고 execute때 값을 사용한다고 전제하는것이 안전하다.
 * 또하나의 주의사항은 각각의 값은 값이 다를 경우 별도의 메모리 공간에 잇어야한다.
 * i라는 변수에 3이라는 값을 넣고 0번 index에 바인딩하고, 다시 i라는 변수에 5를 넣고 1번 index에 바인딩할경우 두값 모두 5가 된다.
 * 이또한 프로바이더마다 다를수있으나 설명한 내용대로 할경우 안전하다.
 * 간략히 설명하자면 이시점에서 파라미터정보는 포인터만을 가지고 있으며 execute시에 해당 포인터의 값을 바인딩 한다고 보면 된다. 
 * 
 * Return value: 오류. 0: 정상
 */
OtDbParamErr
ot_db_dataset_set_param (OtDbDataset    *dataset,
                         gint            index,
                         OtDbCType       ctype,
                         gpointer        dbtype,
                         gpointer        value,
                         gint            length)
{
    OtDbDatasetClass *class = OT_DB_DATASET_GET_CLASS (dataset);

    return class->set_param (dataset, index, ctype, dbtype, value, length);
}

gboolean
ot_db_dataset_execute (OtDbDataset *dataset)
{
    OtDbDatasetClass *class = OT_DB_DATASET_GET_CLASS (dataset);

    return class->execute (dataset);
}

gboolean
ot_db_dataset_close (OtDbDataset *dataset)
{
    OtDbDatasetClass *class = OT_DB_DATASET_GET_CLASS (dataset);

    ot_db_dataset_clear (dataset);

    return class->close (dataset);
}

GPtrArray*
ot_db_dataset_dup_result_columns (OtDbDataset *dataset)
{
    OtDbDatasetClass *class = OT_DB_DATASET_GET_CLASS (dataset);

    return class->dup_result_columns (dataset);
}

gboolean
ot_db_dataset_result_next (OtDbDataset *dataset)
{
    OtDbDatasetClass    *class  = OT_DB_DATASET_GET_CLASS (dataset);
    gboolean             next   = class->result_next (dataset);

    if (next)
    {
        dataset->req_cursor_index++;
    }

    return next;
}

gboolean
ot_db_dataset_result_move_cursor (OtDbDataset   *dataset,
                                  gint           row_index)
{
    OtDbDatasetClass *class  = OT_DB_DATASET_GET_CLASS (dataset);

    return class->result_move_cursor (dataset, row_index);
}

GPtrArray*
ot_db_dataset_dup_result_data (OtDbDataset  *dataset,
                               gboolean      string_only)
{
    OtDbDatasetClass *class = OT_DB_DATASET_GET_CLASS (dataset);

    return class->dup_result_data (dataset, string_only);
}

gboolean
ot_db_dataset_update_result_datas (OtDbDataset      *dataset,
                                   gint             *row_indexes,
                                   GPtrArray        *row_datas,
                                   gchar            *table_name)
{
    OtDbDatasetClass *class = OT_DB_DATASET_GET_CLASS (dataset);

    return class->update_result_datas (dataset, row_indexes, row_datas,
            table_name);
}

gboolean
ot_db_dataset_delete_result_datas (OtDbDataset      *dataset,
                                   gint             *row_indexes,
                                   gint              row_count,
                                   gchar            *table_name)
{
    OtDbDatasetClass *class = OT_DB_DATASET_GET_CLASS (dataset);

    return class->delete_result_datas (dataset, row_indexes, row_count,
            table_name);
}

gpointer
ot_db_dataset_get_result_dbtype (OtDbDataset    *dataset,
                                 gint            column_index)
{
    OtDbDatasetClass *class = OT_DB_DATASET_GET_CLASS (dataset);

    return class->get_result_dbtype (dataset, column_index);
}

void
ot_db_dataset_clear (OtDbDataset *dataset)
{
    if (dataset->req_columns != NULL)
    {
        gint i = 0;
        for (i = 0; i < dataset->req_columns->len; i++)
        {
            OtDbColumn *column = g_ptr_array_index (dataset->req_columns, i);
            if (column != NULL)
            {
                g_object_unref (column);
            }
        }
        g_ptr_array_free (dataset->req_columns, TRUE);
        dataset->req_columns = NULL;
    }

    dataset->req_stmt_type       = OT_DB_STMT_TYPE_NONE;
    dataset->req_modify_count    = 0;
    dataset->req_cursor_index    = -1;
}

GPtrArray*
ot_db_dataset_free_result_data (OtDbDataset *dataset,
                                GPtrArray   *data_array)
{
    GPtrArray *columns = dataset->req_columns; 

    if (columns == NULL || data_array == NULL)
    {
        return NULL;
    }

    gint i = 0;
    for (i = 0; i < data_array->len; i++)
    {
        OtDbColumn  *column = g_ptr_array_index (columns, i);
        gpointer    *value  = g_ptr_array_index (data_array, i);

        if (value == NULL)
        {
            continue;
        }

        switch (column->ctype)
        {
            case OT_DB_C_TYPE_STR_BUF:
                g_free (((OtDbStrBuf *) value)->data);
            break;

            default:
            break;
        }

        g_free (value);
    }

    g_ptr_array_free (data_array, TRUE);

    return NULL;
}
