/* 파일 설명 적는곳

   This file is part of otak-dbmanager.

   Copyright (C) 2007 Otak-dbmanager Project.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
*/

#include <string.h>
#include <glib.h>

#include "../otak-util/otstring.h"
#include "otvalue.h"

OtValue*
ot_value_new (GtkWidget         *widget,
              const gchar       *name,
              const gchar       *default_value,
              OtValueValidFlag   valid_flags)
{
    OtValue *value = g_malloc0 (sizeof (OtValue));

    value->widget           = widget;
    value->name             = g_strdup (name);
    value->default_value    = NULL;
    value->valid_flags      = valid_flags;
    value->min              = 0;
    value->max              = 0;

    if (default_value != NULL)
    {
        value->default_value = g_strdup (default_value);
    }
    
    return value;
}

OtValue*
ot_value_free (OtValue *otvalue)
{
    otvalue->widget         = NULL;
    otvalue->valid_flags    = OT_VALUE_VALID_NONE;

    g_free (otvalue->name);
    otvalue->name = NULL;

    if (otvalue->default_value != NULL)
    {
        g_free (otvalue->default_value);
        otvalue->default_value = NULL;
    }

    g_free (otvalue);

    return NULL;
}

OtValueValidFlag
ot_value_get_valid_err (OtValue *otvalue)
{
    OtValueValidFlag     return_flags   = OT_VALUE_VALID_NONE;
    OtValueValidFlag     flags          = otvalue->valid_flags;
    const gchar         *value          = ot_value_get_value (otvalue);
    gint                 value_len      = 0;

    if (ot_string_is_empty (value))
    {
        if (flags & OT_VALUE_VALID_REQUIRE)
        {
            return_flags = return_flags | OT_VALUE_VALID_REQUIRE;
        }

        value_len = 0;
    }
    else
    {
        if (flags & OT_VALUE_VALID_NUMERIC && !ot_string_is_numeric (value))
        {
            return_flags = return_flags | OT_VALUE_VALID_NUMERIC;
        }
    
        if (flags & OT_VALUE_VALID_ALPHA && !ot_string_is_alpha (value))
        {
            return_flags = return_flags | OT_VALUE_VALID_ALPHA;
        }

        if (flags & OT_VALUE_VALID_ALPHANUM && !ot_string_is_alphanum (value))
        {
            return_flags = return_flags | OT_VALUE_VALID_ALPHANUM;
        }

        if (flags & OT_VALUE_VALID_ONEBYTE && !ot_string_is_onebyte (value))
        {
            return_flags = return_flags | OT_VALUE_VALID_ONEBYTE;
        }

        value_len = strlen (value);
    }

    if (flags & OT_VALUE_VALID_LENGTH
            && (otvalue->min < value_len || (otvalue->max > 0 && otvalue->max > value_len)))
    {
        return_flags = return_flags | OT_VALUE_VALID_LENGTH;
    }

    return return_flags;
}

const gchar*
ot_value_get_value (OtValue *otvalue)
{
    GtkWidget   *widget = otvalue->widget;
    const gchar *value  = NULL;

    if (GTK_IS_ENTRY (widget))
    {
        value = gtk_entry_get_text (GTK_ENTRY (widget));
    }

    return value;
}

void
ot_value_set_value (OtValue     *otvalue,
                    const gchar *value)
{
    GtkWidget   *widget         = otvalue->widget;
    const gchar *default_value  = otvalue->default_value; 

    if (GTK_IS_ENTRY (widget))
    {
        const gchar *value_final = value;
        if (value_final == NULL)
        {
            if (default_value == NULL)
            {
                value_final = "";
            }
            else
            {
                value_final = default_value;
            }
        }
        gtk_entry_set_text (GTK_ENTRY (widget), value_final);
    }
}

void
ot_value_set_length (OtValue    *otvalue,
                     gint        min,
                     gint        max)
{
    otvalue->min    = min;
    otvalue->max    = max;
}
